import { scaffolderPlugin } from '@backstage/plugin-scaffolder';
import { createScaffolderFieldExtension } from '@backstage/plugin-scaffolder-react';
import React, { useState, useCallback } from 'react';
import FormControl from '@material-ui/core/FormControl';
import { useApi, microsoftAuthApiRef, githubAuthApiRef, googleAuthApiRef, bitbucketAuthApiRef, identityApiRef, discoveryApiRef, fetchApiRef } from '@backstage/core-plugin-api';
import { ErrorPanel, Progress, Select } from '@backstage/core-components';
import { useAsync } from 'react-use';
import get from 'lodash/get';
import sortBy from 'lodash/sortBy';
import { z } from 'zod';
import { FormHelperText, Box, Typography, Button } from '@material-ui/core';
import { renderString } from 'nunjucks';
import fromPairs from 'lodash/fromPairs';

const selectFieldFromApiConfigSchema = z.object({
  params: z.record(z.string(), z.string()).optional(),
  path: z.string(),
  arraySelector: z.string().or(z.array(z.string())).optional(),
  valueSelector: z.string().or(z.array(z.string())).optional(),
  labelSelector: z.string().or(z.array(z.string())).optional(),
  title: z.string().optional(),
  placeholder: z.string().optional(),
  description: z.string().optional(),
  oauth: z.object({
    provider: z.enum(["microsoft", "github", "bitbucket", "google"]),
    scopes: z.array(z.string())
  }).optional()
});

const oauthApiRefMap = {
  microsoft: microsoftAuthApiRef,
  github: githubAuthApiRef,
  google: googleAuthApiRef,
  bitbucket: bitbucketAuthApiRef
};
const useOauthSignIn = ({ provider, scopes }) => {
  const authApi = useApi(oauthApiRefMap[provider != null ? provider : "github"]);
  const [isSignedIn, setSignedIn] = useState(false);
  const [token, setToken] = useState("");
  const [forceLogin, setForceLogin] = useState(false);
  const doSignIn = useCallback(
    async (usePopup = true) => {
      const accessToken = await authApi.getAccessToken(scopes, {
        optional: !usePopup,
        instantPopup: usePopup
      });
      if (accessToken) {
        setSignedIn(!!accessToken);
        setToken(accessToken);
      }
    },
    [authApi, setSignedIn, scopes]
  );
  const { loading, error } = useAsync(async () => {
    if (!isSignedIn) {
      await doSignIn(forceLogin);
    }
  }, [doSignIn, forceLogin, isSignedIn]);
  const showSignInModal = useCallback(() => {
    setForceLogin(true);
  }, [setForceLogin]);
  return { token, loading, error, isSignedIn, showSignInModal };
};

const renderOption = (input, context) => {
  if (!input) {
    return input;
  }
  if (typeof input === "string") {
    return renderString(input, context);
  }
  if (Array.isArray(input)) {
    return input.map((item) => renderOption(item, context));
  }
  if (typeof input === "object") {
    return fromPairs(
      Object.keys(input).map((key) => [
        key,
        renderOption(input[key], context)
      ])
    );
  }
  return input;
};
const SelectFieldFromApiComponent = (props) => {
  var _a;
  const discoveryApi = useApi(discoveryApiRef);
  const fetchApi = useApi(fetchApiRef);
  const [dropDownData, setDropDownData] = useState();
  const { formContext, uiSchema, identity } = props;
  const optionsParsingState = selectFieldFromApiConfigSchema.safeParse(
    uiSchema["ui:options"]
  );
  const { error } = useAsync(async () => {
    if (!optionsParsingState.success) {
      throw optionsParsingState.error;
    }
    const options = optionsParsingState.data;
    const baseUrl = await discoveryApi.getBaseUrl("");
    const headers = {};
    if (props.token) {
      headers.Authorization = `Bearer ${props.token}`;
    }
    const params = new URLSearchParams(
      renderOption(options.params, {
        parameters: formContext.formData,
        identity
      })
    );
    const response = await fetchApi.fetch(
      `${baseUrl}${renderOption(options.path, {
        parameters: formContext.formData
      })}?${params}`,
      { headers }
    );
    const body = await response.json();
    const array = options.arraySelector ? get(
      body,
      renderOption(options.arraySelector, {
        parameters: formContext.formData
      })
    ) : body;
    const constructedData = array.map((item) => {
      let value;
      let label;
      if (options.valueSelector) {
        value = get(
          item,
          renderOption(options.valueSelector, {
            parameters: formContext.formData
          })
        );
        label = options.labelSelector ? get(
          item,
          renderOption(options.labelSelector, {
            parameters: formContext.formData
          })
        ) : value;
      } else {
        if (!(typeof item === "string")) {
          throw new Error(
            `The item provided for the select drop down "${item}" is not a string`
          );
        }
        value = item;
        label = item;
      }
      if (!value) {
        throw new Error(`Failed to populate SelectFieldFromApi dropdown`);
      }
      return {
        value,
        label: label || value
      };
    });
    setDropDownData(sortBy(constructedData, "label"));
  });
  const {
    title = "Select",
    description = "",
    placeholder = "Select from results"
  } = optionsParsingState.success ? optionsParsingState.data : {};
  if (error) {
    return /* @__PURE__ */ React.createElement(ErrorPanel, { error });
  }
  if (!dropDownData) {
    return /* @__PURE__ */ React.createElement(Progress, null);
  }
  return /* @__PURE__ */ React.createElement(
    FormControl,
    {
      margin: "normal",
      required: props.required,
      error: (((_a = props == null ? void 0 : props.rawErrors) == null ? void 0 : _a.length) || 0) > 0 && !props.formData
    },
    /* @__PURE__ */ React.createElement(
      Select,
      {
        items: dropDownData,
        placeholder,
        label: title,
        onChange: (selected) => {
          props.onChange(
            Array.isArray(selected) && !selected.length ? void 0 : selected.toString()
          );
        }
      }
    ),
    /* @__PURE__ */ React.createElement(FormHelperText, null, description)
  );
};
const SelectFieldFromApiOauthWrapper = ({
  oauthConfig,
  ...props
}) => {
  var _a;
  const { token, loading, error, isSignedIn, showSignInModal } = useOauthSignIn(oauthConfig);
  if (loading && !isSignedIn) {
    return /* @__PURE__ */ React.createElement(Progress, null);
  }
  if (error) {
    return /* @__PURE__ */ React.createElement(ErrorPanel, { error });
  }
  if (!props.uiSchema) {
    return /* @__PURE__ */ React.createElement(ErrorPanel, { error: new Error("No UI Schema defined") });
  }
  if (!isSignedIn || !token) {
    return /* @__PURE__ */ React.createElement(Box, { height: "100%", width: "100%" }, /* @__PURE__ */ React.createElement(Box, null, /* @__PURE__ */ React.createElement(Typography, { variant: "body2" }, /* @__PURE__ */ React.createElement("b", null, ((_a = props.uiSchema["ui:options"]) == null ? void 0 : _a.title) || props.name))), /* @__PURE__ */ React.createElement(Box, { display: "flex" }, /* @__PURE__ */ React.createElement(Box, { paddingRight: 1 }, /* @__PURE__ */ React.createElement(Typography, null, "This input requires authentication with ", oauthConfig.provider)), /* @__PURE__ */ React.createElement(Box, null, /* @__PURE__ */ React.createElement(
      Button,
      {
        variant: "outlined",
        color: "primary",
        onClick: showSignInModal,
        size: "small"
      },
      "Sign In"
    ))));
  }
  return /* @__PURE__ */ React.createElement(
    SelectFieldFromApiComponent,
    {
      ...{ ...props, uiSchema: props.uiSchema },
      token
    }
  );
};
const SelectFieldFromApi = (props) => {
  const identityApi = useApi(identityApiRef);
  const { loading, value: identity } = useAsync(async () => {
    return await identityApi.getBackstageIdentity();
  });
  if (!props.uiSchema) {
    return /* @__PURE__ */ React.createElement(ErrorPanel, { error: new Error("No UI Schema defined") });
  }
  const result = selectFieldFromApiConfigSchema.safeParse(
    props.uiSchema["ui:options"]
  );
  if (loading) {
    return null;
  }
  if (result.success && result.data.oauth) {
    return /* @__PURE__ */ React.createElement(
      SelectFieldFromApiOauthWrapper,
      {
        identity,
        oauthConfig: result.data.oauth,
        ...props
      }
    );
  }
  return /* @__PURE__ */ React.createElement(
    SelectFieldFromApiComponent,
    {
      ...{ ...props, uiSchema: props.uiSchema },
      identity
    }
  );
};

const SelectFieldFromApiExtension = scaffolderPlugin.provide(
  createScaffolderFieldExtension({
    name: "SelectFieldFromApi",
    component: SelectFieldFromApi
  })
);

export { SelectFieldFromApiExtension };
//# sourceMappingURL=index.esm.js.map
